/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSImageRep;

import dstep.appkit.AppKitDefines;
import dstep.appkit.NSPasteboard;
import dstep.foundation.NSArray;
import dstep.foundation.NSCoder;
import dstep.foundation.NSData;
import dstep.foundation.NSGeometry;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSURL;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

import bindings = dstep.appkit.NSImageRep_bindings;

private
{
	NSString NSImageRepRegistryDidChangeNotification_;
}

NSString NSImageRepRegistryDidChangeNotification ()
{
	if (NSImageRepRegistryDidChangeNotification_)
		return NSImageRepRegistryDidChangeNotification_;

	return NSImageRepRegistryDidChangeNotification_ = new NSString(bindings.NSImageRepRegistryDidChangeNotification);
}

enum
{
	NSImageRepMatchesDevice = 0
}

class NSImageRep : NSObject, INSCopying, INSCoding
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this (NSCoder aDecoder)
	{
		super(typeof(this).alloc.initWithCoder(aDecoder).objcObject);
	}
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	typeof(this) initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(typeof(this), "initWithCoder:", NSCoder)(aDecoder);
	}
	
	typeof(this) copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(typeof(this), "copyWithZone:", NSZone*)(zone);
	}

	bool draw ()
	{
		return invokeObjcSelf!(bool, "draw");
	}

	bool drawAtPoint (NSPoint point)
	{
		return invokeObjcSelf!(bool, "drawAtPoint:", NSPoint)(point);
	}

	bool drawInRect (NSRect rect)
	{
		return invokeObjcSelf!(bool, "drawInRect:", NSRect)(rect);
	}

	void setSize (NSSize aSize)
	{
		return invokeObjcSelf!(void, "setSize:", NSSize)(aSize);
	}

	NSSize size ()
	{
		return invokeObjcSelf!(NSSize, "size");
	}

	void setAlpha (bool flag)
	{
		return invokeObjcSelf!(void, "setAlpha:", bool)(flag);
	}

	bool hasAlpha ()
	{
		return invokeObjcSelf!(bool, "hasAlpha");
	}

	void setOpaque (bool flag)
	{
		return invokeObjcSelf!(void, "setOpaque:", bool)(flag);
	}

	bool isOpaque ()
	{
		return invokeObjcSelf!(bool, "isOpaque");
	}

	void setColorSpaceName (NSString string)
	{
		return invokeObjcSelf!(void, "setColorSpaceName:", NSString)(string);
	}

	NSString colorSpaceName ()
	{
		return invokeObjcSelf!(NSString, "colorSpaceName");
	}

	void setBitsPerSample (NSInteger anInt)
	{
		return invokeObjcSelf!(void, "setBitsPerSample:", NSInteger)(anInt);
	}

	NSInteger bitsPerSample ()
	{
		return invokeObjcSelf!(NSInteger, "bitsPerSample");
	}

	void setPixelsWide (NSInteger anInt)
	{
		return invokeObjcSelf!(void, "setPixelsWide:", NSInteger)(anInt);
	}

	NSInteger pixelsWide ()
	{
		return invokeObjcSelf!(NSInteger, "pixelsWide");
	}

	void setPixelsHigh (NSInteger anInt)
	{
		return invokeObjcSelf!(void, "setPixelsHigh:", NSInteger)(anInt);
	}

	NSInteger pixelsHigh ()
	{
		return invokeObjcSelf!(NSInteger, "pixelsHigh");
	}

	static void registerImageRepClass (Class imageRepClass)
	{
		return invokeObjcSelfClass!(void, "registerImageRepClass:", Class)(imageRepClass);
	}

	static void unregisterImageRepClass (Class imageRepClass)
	{
		return invokeObjcSelfClass!(void, "unregisterImageRepClass:", Class)(imageRepClass);
	}

	static NSArray registeredImageRepClasses ()
	{
		return invokeObjcSelfClass!(NSArray, "registeredImageRepClasses");
	}

	static Class imageRepClassForFileType (NSString type)
	{
		return invokeObjcSelfClass!(Class, "imageRepClassForFileType:", NSString)(type);
	}

	static Class imageRepClassForPasteboardType (NSString type)
	{
		return invokeObjcSelfClass!(Class, "imageRepClassForPasteboardType:", NSString)(type);
	}

	static Class imageRepClassForType (NSString type)
	{
		return invokeObjcSelfClass!(Class, "imageRepClassForType:", NSString)(type);
	}

	static Class imageRepClassForData (NSData data)
	{
		return invokeObjcSelfClass!(Class, "imageRepClassForData:", NSData)(data);
	}

	static bool canInitWithData (NSData data)
	{
		return invokeObjcSelfClass!(bool, "canInitWithData:", NSData)(data);
	}

	static NSArray imageUnfilteredFileTypes ()
	{
		return invokeObjcSelfClass!(NSArray, "imageUnfilteredFileTypes");
	}

	static NSArray imageUnfilteredPasteboardTypes ()
	{
		return invokeObjcSelfClass!(NSArray, "imageUnfilteredPasteboardTypes");
	}

	static NSArray imageFileTypes ()
	{
		return invokeObjcSelfClass!(NSArray, "imageFileTypes");
	}

	static NSArray imagePasteboardTypes ()
	{
		return invokeObjcSelfClass!(NSArray, "imagePasteboardTypes");
	}

	static NSArray imageUnfilteredTypes ()
	{
		return invokeObjcSelfClass!(NSArray, "imageUnfilteredTypes");
	}

	static NSArray imageTypes ()
	{
		return invokeObjcSelfClass!(NSArray, "imageTypes");
	}

	static bool canInitWithPasteboard (NSPasteboard pasteboard)
	{
		return invokeObjcSelfClass!(bool, "canInitWithPasteboard:", NSPasteboard)(pasteboard);
	}

	static NSArray imageRepsWithContentsOfFile (NSString filename)
	{
		return invokeObjcSelfClass!(NSArray, "imageRepsWithContentsOfFile:", NSString)(filename);
	}

	static Object imageRepWithContentsOfFile (NSString filename)
	{
		return invokeObjcSelfClass!(Object, "imageRepWithContentsOfFile:", NSString)(filename);
	}

	static NSArray imageRepsWithContentsOfURL (NSURL url)
	{
		return invokeObjcSelfClass!(NSArray, "imageRepsWithContentsOfURL:", NSURL)(url);
	}

	static Object imageRepWithContentsOfURL (NSURL url)
	{
		return invokeObjcSelfClass!(Object, "imageRepWithContentsOfURL:", NSURL)(url);
	}

	static NSArray imageRepsWithPasteboard (NSPasteboard pasteboard)
	{
		return invokeObjcSelfClass!(NSArray, "imageRepsWithPasteboard:", NSPasteboard)(pasteboard);
	}

	static Object imageRepWithPasteboard (NSPasteboard pasteboard)
	{
		return invokeObjcSelfClass!(Object, "imageRepWithPasteboard:", NSPasteboard)(pasteboard);
	}

}

