/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 25, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.coredata.NSFetchRequest;

import dstep.coredata.NSEntityDescription;
import dstep.coredata.NSManagedObjectModel;
import dstep.foundation.NSArray;
import dstep.foundation.NSCoder;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSPredicate;
import dstep.foundation.NSString;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

alias NSUInteger NSFetchRequestResultType;

enum
{
	NSManagedObjectResultType = 0x00,
	NSManagedObjectIDResultType = 0x01
}

class NSFetchRequest : NSObject, INSCoding, INSCopying
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}

	NSEntityDescription entity ()
	{
		return invokeObjcSelf!(NSEntityDescription, "entity");
	}

	void setEntity (NSEntityDescription entity)
	{
		return invokeObjcSelf!(void, "setEntity:", NSEntityDescription)(entity);
	}

	NSPredicate predicate ()
	{
		return invokeObjcSelf!(NSPredicate, "predicate");
	}

	void setPredicate (NSPredicate predicate)
	{
		return invokeObjcSelf!(void, "setPredicate:", NSPredicate)(predicate);
	}

	NSArray sortDescriptors ()
	{
		return invokeObjcSelf!(NSArray, "sortDescriptors");
	}

	void setSortDescriptors (NSArray sortDescriptors)
	{
		return invokeObjcSelf!(void, "setSortDescriptors:", NSArray)(sortDescriptors);
	}

	NSUInteger fetchLimit ()
	{
		return invokeObjcSelf!(NSUInteger, "fetchLimit");
	}

	void setFetchLimit (NSUInteger limit)
	{
		return invokeObjcSelf!(void, "setFetchLimit:", NSUInteger)(limit);
	}

	NSArray affectedStores ()
	{
		return invokeObjcSelf!(NSArray, "affectedStores");
	}

	void setAffectedStores (NSArray stores)
	{
		return invokeObjcSelf!(void, "setAffectedStores:", NSArray)(stores);
	}

	uint resultType ()
	{
		return invokeObjcSelf!(uint, "resultType");
	}

	void setResultType (uint type)
	{
		return invokeObjcSelf!(void, "setResultType:", uint)(type);
	}

	bool includesSubentities ()
	{
		return invokeObjcSelf!(bool, "includesSubentities");
	}

	void setIncludesSubentities (bool yesNo)
	{
		return invokeObjcSelf!(void, "setIncludesSubentities:", bool)(yesNo);
	}

	bool includesPropertyValues ()
	{
		return invokeObjcSelf!(bool, "includesPropertyValues");
	}

	void setIncludesPropertyValues (bool yesNo)
	{
		return invokeObjcSelf!(void, "setIncludesPropertyValues:", bool)(yesNo);
	}

	bool returnsObjectsAsFaults ()
	{
		return invokeObjcSelf!(bool, "returnsObjectsAsFaults");
	}

	void setReturnsObjectsAsFaults (bool yesNo)
	{
		return invokeObjcSelf!(void, "setReturnsObjectsAsFaults:", bool)(yesNo);
	}

	NSArray relationshipKeyPathsForPrefetching ()
	{
		return invokeObjcSelf!(NSArray, "relationshipKeyPathsForPrefetching");
	}

	void setRelationshipKeyPathsForPrefetching (NSArray keys)
	{
		return invokeObjcSelf!(void, "setRelationshipKeyPathsForPrefetching:", NSArray)(keys);
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	typeof(this) initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(typeof(this), "initWithCoder:", NSCoder)(aDecoder);
	}
	
	typeof(this) copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(typeof(this), "copyWithZone:", NSZone*)(zone);
	}
}

