/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 25, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.coredata.NSManagedObjectModel;

import dstep.coredata.NSEntityDescription;
import dstep.coredata.NSFetchRequest;
import dstep.foundation.NSArray;
import dstep.foundation.NSCoder;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSEnumerator;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSSet;
import dstep.foundation.NSString;
import dstep.foundation.NSURL;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSManagedObjectModel : NSObject, INSCoding, INSCopying, INSFastEnumeration
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}

	static NSManagedObjectModel mergedModelFromBundles (NSArray bundles)
	{
		return invokeObjcSelfClass!(NSManagedObjectModel, "mergedModelFromBundles:", NSArray)(bundles);
	}

	static NSManagedObjectModel modelByMergingModels (NSArray models)
	{
		return invokeObjcSelfClass!(NSManagedObjectModel, "modelByMergingModels:", NSArray)(models);
	}

	NSManagedObjectModel init ()
	{
		id result = invokeObjcSelf!(id, "init");
		return result is this.objcObject ? this : (result !is null ? new NSManagedObjectModel(result) : null);
	}

	this ()
	{
		super(NSManagedObjectModel.alloc.init.objcObject);
	}

	NSManagedObjectModel initWithContentsOfURL (NSURL url)
	{
		id result = invokeObjcSelf!(id, "initWithContentsOfURL:", NSURL)(url);
		return result is this.objcObject ? this : (result !is null ? new NSManagedObjectModel(result) : null);
	}

	this (NSURL url)
	{
		super(NSManagedObjectModel.alloc.initWithContentsOfURL(url).objcObject);
	}

	NSDictionary entitiesByName ()
	{
		return invokeObjcSelf!(NSDictionary, "entitiesByName");
	}

	NSArray entities ()
	{
		return invokeObjcSelf!(NSArray, "entities");
	}

	void setEntities (NSArray entities)
	{
		return invokeObjcSelf!(void, "setEntities:", NSArray)(entities);
	}

	NSArray configurations ()
	{
		return invokeObjcSelf!(NSArray, "configurations");
	}

	NSArray entitiesForConfiguration (NSString configuration)
	{
		return invokeObjcSelf!(NSArray, "entitiesForConfiguration:", NSString)(configuration);
	}

	void setEntities (NSArray entities, NSString configuration)
	{
		return invokeObjcSelf!(void, "setEntities:forConfiguration:", NSArray, NSString)(entities, configuration);
	}

	void setFetchRequestTemplate (NSFetchRequest fetchRequestTemplate, NSString name)
	{
		return invokeObjcSelf!(void, "setFetchRequestTemplate:forName:", NSFetchRequest, NSString)(fetchRequestTemplate, name);
	}

	NSFetchRequest fetchRequestTemplateForName (NSString name)
	{
		return invokeObjcSelf!(NSFetchRequest, "fetchRequestTemplateForName:", NSString)(name);
	}

	NSFetchRequest fetchRequestFromTemplateWithName (NSString name, NSDictionary variables)
	{
		return invokeObjcSelf!(NSFetchRequest, "fetchRequestFromTemplateWithName:substitutionVariables:", NSString, NSDictionary)(name, variables);
	}

	NSDictionary localizationDictionary ()
	{
		return invokeObjcSelf!(NSDictionary, "localizationDictionary");
	}

	void setLocalizationDictionary (NSDictionary localizationDictionary)
	{
		return invokeObjcSelf!(void, "setLocalizationDictionary:", NSDictionary)(localizationDictionary);
	}

	static NSManagedObjectModel mergedModelFromBundles (NSArray bundles, NSDictionary metadata)
	{
		return invokeObjcSelfClass!(NSManagedObjectModel, "mergedModelFromBundles:forStoreMetadata:", NSArray, NSDictionary)(bundles, metadata);
	}

	static NSManagedObjectModel modelByMergingModels (NSArray models, NSDictionary metadata)
	{
		return invokeObjcSelfClass!(NSManagedObjectModel, "modelByMergingModels:forStoreMetadata:", NSArray, NSDictionary)(models, metadata);
	}

	NSDictionary fetchRequestTemplatesByName ()
	{
		return invokeObjcSelf!(NSDictionary, "fetchRequestTemplatesByName");
	}

	NSSet versionIdentifiers ()
	{
		return invokeObjcSelf!(NSSet, "versionIdentifiers");
	}

	void setVersionIdentifiers (NSSet identifiers)
	{
		return invokeObjcSelf!(void, "setVersionIdentifiers:", NSSet)(identifiers);
	}

	bool isConfiguration (NSString configuration, NSDictionary metadata)
	{
		return invokeObjcSelf!(bool, "isConfiguration:compatibleWithStoreMetadata:", NSString, NSDictionary)(configuration, metadata);
	}

	NSDictionary entityVersionHashesByName ()
	{
		return invokeObjcSelf!(NSDictionary, "entityVersionHashesByName");
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	typeof(this) initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(typeof(this), "initWithCoder:", NSCoder)(aDecoder);
	}
	
	typeof(this) copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(typeof(this), "copyWithZone:", NSZone*)(zone);
	}
	
	NSUInteger countByEnumeratingWithState (NSFastEnumerationState* state, id* stackbuf, NSUInteger len)
	{
		return invokeObjcSelf!(NSUInteger, "countByEnumeratingWithState:objects:count:", NSFastEnumerationState*, id*, NSUInteger)(state, stackbuf, len);
	}
}

