/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSNotification;

import dstep.foundation.NSCoder;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSObject;
import dstep.foundation.NSString;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

const TNSNotificationCreation = `

	static Object notificationWithName (NSString aName, Object anObject)
	{
		return invokeObjcSuperClass!(Object, "notificationWithName:object:", NSString, Object)(aName, anObject);
	}

	static Object notificationWithName (NSString aName, Object anObject, NSDictionary aUserInfo)
	{
		return invokeObjcSuperClass!(Object, "notificationWithName:object:userInfo:", NSString, Object, NSDictionary)(aName, anObject, aUserInfo);
	}
`;

class NSNotification : NSObject, INSCopying, INSCoding
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	NSString name ()
	{
		return invokeObjcSelf!(NSString, "name");
	}

	Object object ()
	{
		return invokeObjcSelf!(Object, "object");
	}

	NSDictionary userInfo ()
	{
		return invokeObjcSelf!(NSDictionary, "userInfo");
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}

	Object initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(Object, "initWithCoder:", NSCoder)(aDecoder);
	}

	this (NSCoder aDecoder)
	{
		typeof(this).alloc.initWithCoder(aDecoder);
	}
	
	// TNSNotificationCreation
	static Object notificationWithName (NSString aName, Object anObject)
	{
		return invokeObjcSuperClass!(Object, "notificationWithName:object:", NSString, Object)(aName, anObject);
	}
	
	static Object notificationWithName (NSString aName, Object anObject, NSDictionary aUserInfo)
	{
		return invokeObjcSuperClass!(Object, "notificationWithName:object:userInfo:", NSString, Object, NSDictionary)(aName, anObject, aUserInfo);
	}
}

class NSNotificationCenter : NSObject
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	static Object defaultCenter ()
	{
		return invokeObjcSuperClass!(Object, "defaultCenter");
	}

	void addObserver (Object observer, SEL aSelector, NSString aName, Object anObject)
	{
		return invokeObjcSelf!(void, "addObserver:selector:name:object:", Object, SEL, NSString, Object)(observer, aSelector, aName, anObject);
	}

	void postNotification (NSNotification notification)
	{
		return invokeObjcSelf!(void, "postNotification:", NSNotification)(notification);
	}

	void postNotificationName (NSString aName, Object anObject)
	{
		return invokeObjcSelf!(void, "postNotificationName:object:", NSString, Object)(aName, anObject);
	}

	void postNotificationName (NSString aName, Object anObject, NSDictionary aUserInfo)
	{
		return invokeObjcSelf!(void, "postNotificationName:object:userInfo:", NSString, Object, NSDictionary)(aName, anObject, aUserInfo);
	}

	void removeObserver (Object observer)
	{
		return invokeObjcSelf!(void, "removeObserver:", Object)(observer);
	}

	void removeObserver (Object observer, NSString aName, Object anObject)
	{
		return invokeObjcSelf!(void, "removeObserver:name:object:", Object, NSString, Object)(observer, aName, anObject);
	}
}