/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSPredicate;

import dstep.foundation.NSArray;
import dstep.foundation.NSCoder;
import dstep.foundation.NSDictionary;
import dstep.foundation.NSObject;
import dstep.foundation.NSSet;
import dstep.foundation.NSString;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;


const TNSPredicateSupport = `

	void filterUsingPredicate (NSPredicate predicate)
	{
		return invokeObjcSelf!(void, "filterUsingPredicate:", NSPredicate)(predicate);
	}
`;

class NSPredicate : NSObject, INSCoding, INSCopying
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}
	
	this ()
	{
		super(typeof(this).alloc.init.objcObject);
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	static NSPredicate predicateWithFormat (NSString predicateFormat, NSArray arguments)
	{
		return invokeObjcSuperClass!(NSPredicate, "predicateWithFormat:argumentArray:", NSString, NSArray)(predicateFormat, arguments);
	}

	static NSPredicate predicateWithFormat (NSString predicateWithFormat, ...)
	{
		return invokeObjcSuperClass!(NSPredicate, "predicateWithFormat:", NSString)(predicateWithFormat);
	}

	static NSPredicate predicateWithFormat (NSString predicateFormat, char* argList)
	{
		return invokeObjcSuperClass!(NSPredicate, "predicateWithFormat:arguments:", NSString, char*)(predicateFormat, argList);
	}

	static NSPredicate predicateWithValue (bool value)
	{
		return invokeObjcSuperClass!(NSPredicate, "predicateWithValue:", bool)(value);
	}

	NSString predicateFormat ()
	{
		return invokeObjcSelf!(NSString, "predicateFormat");
	}

	NSPredicate predicateWithSubstitutionVariables (NSDictionary variables)
	{
		id result = invokeObjcSelf!(id, "predicateWithSubstitutionVariables:", NSDictionary)(variables);
		return result is this.objcObject ? this : (result !is null ? new NSPredicate(result) : null);
	}

	bool evaluateWithObject (Object object)
	{
		return invokeObjcSelf!(bool, "evaluateWithObject:", Object)(object);
	}

	bool evaluateWithObject (Object object, NSDictionary bindings)
	{
		return invokeObjcSelf!(bool, "evaluateWithObject:substitutionVariables:", Object, NSDictionary)(object, bindings);
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}

	Object initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(Object, "initWithCoder:", NSCoder)(aDecoder);
	}

	this (NSCoder aDecoder)
	{
		typeof(this).alloc.initWithCoder(aDecoder);
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}
}