/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 28, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.qtkit.QTDataReference;

import dstep.coreservices.carboncore.MacTypes;
import dstep.foundation.Foundation;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;
import dstep.qtkit.QTKitDefines;

import bindings = dstep.qtkit.QTDataReference_bindings;

private
{
	NSString QTDataReferenceTypeFile_;
	NSString QTDataReferenceTypeHandle_;
	NSString QTDataReferenceTypePointer_;
	NSString QTDataReferenceTypeResource_;
	NSString QTDataReferenceTypeURL_;
}

NSString QTDataReferenceTypeFile ()
{
	if (QTDataReferenceTypeFile_)
		return QTDataReferenceTypeFile_;

	return QTDataReferenceTypeFile_ = new NSString(bindings.QTDataReferenceTypeFile);
}

NSString QTDataReferenceTypeHandle ()
{
	if (QTDataReferenceTypeHandle_)
		return QTDataReferenceTypeHandle_;

	return QTDataReferenceTypeHandle_ = new NSString(bindings.QTDataReferenceTypeHandle);
}

NSString QTDataReferenceTypePointer ()
{
	if (QTDataReferenceTypePointer_)
		return QTDataReferenceTypePointer_;

	return QTDataReferenceTypePointer_ = new NSString(bindings.QTDataReferenceTypePointer);
}

NSString QTDataReferenceTypeResource ()
{
	if (QTDataReferenceTypeResource_)
		return QTDataReferenceTypeResource_;

	return QTDataReferenceTypeResource_ = new NSString(bindings.QTDataReferenceTypeResource);
}

NSString QTDataReferenceTypeURL ()
{
	if (QTDataReferenceTypeURL_)
		return QTDataReferenceTypeURL_;

	return QTDataReferenceTypeURL_ = new NSString(bindings.QTDataReferenceTypeURL);
}

class QTDataReference : NSObject, INSCoding
{
	mixin ObjcWrap;
	
	this (id object)
	{
		super(object);
	}

	static Object dataReferenceWithDataRef (Handle dataRef, NSString type)
	{
		return invokeObjcSelfClass!(Object, "dataReferenceWithDataRef:type:", Handle, NSString)(dataRef, type);
	}

	static Object dataReferenceWithDataRefData (NSData dataRefData, NSString type)
	{
		return invokeObjcSelfClass!(Object, "dataReferenceWithDataRefData:type:", NSData, NSString)(dataRefData, type);
	}

	static Object dataReferenceWithReferenceToFile (NSString fileName)
	{
		return invokeObjcSelfClass!(Object, "dataReferenceWithReferenceToFile:", NSString)(fileName);
	}

	static Object dataReferenceWithReferenceToURL (NSURL url)
	{
		return invokeObjcSelfClass!(Object, "dataReferenceWithReferenceToURL:", NSURL)(url);
	}

	static Object dataReferenceWithReferenceToData (NSData data)
	{
		return invokeObjcSelfClass!(Object, "dataReferenceWithReferenceToData:", NSData)(data);
	}

	static Object dataReferenceWithReferenceToData (NSData data, NSString name, NSString MIMEType)
	{
		return invokeObjcSelfClass!(Object, "dataReferenceWithReferenceToData:name:MIMEType:", NSData, NSString, NSString)(data, name, MIMEType);
	}

	QTDataReference initWithDataRef (Handle dataRef, NSString type)
	{
		id result = invokeObjcSelf!(id, "initWithDataRef:type:", Handle, NSString)(dataRef, type);
		return result is this.objcObject ? this : (result !is null ? new QTDataReference(result) : null);
	}

	this (Handle dataRef, NSString type)
	{
		super(QTDataReference.alloc.initWithDataRef(dataRef, type).objcObject);
	}

	QTDataReference initWithDataRefData (NSData dataRefData, NSString type)
	{
		id result = invokeObjcSelf!(id, "initWithDataRefData:type:", NSData, NSString)(dataRefData, type);
		return result is this.objcObject ? this : (result !is null ? new QTDataReference(result) : null);
	}

	this (NSData dataRefData, NSString type)
	{
		super(QTDataReference.alloc.initWithDataRefData(dataRefData, type).objcObject);
	}

	QTDataReference initWithReferenceToFile (NSString fileName)
	{
		id result = invokeObjcSelf!(id, "initWithReferenceToFile:", NSString)(fileName);
		return result is this.objcObject ? this : (result !is null ? new QTDataReference(result) : null);
	}

	this (NSString fileName)
	{
		super(QTDataReference.alloc.initWithReferenceToFile(fileName).objcObject);
	}

	QTDataReference initWithReferenceToURL (NSURL url)
	{
		id result = invokeObjcSelf!(id, "initWithReferenceToURL:", NSURL)(url);
		return result is this.objcObject ? this : (result !is null ? new QTDataReference(result) : null);
	}

	this (NSURL url)
	{
		super(QTDataReference.alloc.initWithReferenceToURL(url).objcObject);
	}

	QTDataReference initWithReferenceToData (NSData data)
	{
		id result = invokeObjcSelf!(id, "initWithReferenceToData:", NSData)(data);
		return result is this.objcObject ? this : (result !is null ? new QTDataReference(result) : null);
	}

	this (NSData data)
	{
		super(QTDataReference.alloc.initWithReferenceToData(data).objcObject);
	}

	QTDataReference initWithReferenceToData (NSData data, NSString name, NSString MIMEType)
	{
		id result = invokeObjcSelf!(id, "initWithReferenceToData:name:MIMEType:", NSData, NSString, NSString)(data, name, MIMEType);
		return result is this.objcObject ? this : (result !is null ? new QTDataReference(result) : null);
	}

	this (NSData data, NSString name, NSString MIMEType)
	{
		super(QTDataReference.alloc.initWithReferenceToData(data, name, MIMEType).objcObject);
	}

	Handle dataRef ()
	{
		return invokeObjcSelf!(Handle, "dataRef");
	}

	NSData dataRefData ()
	{
		return invokeObjcSelf!(NSData, "dataRefData");
	}

	NSString dataRefType ()
	{
		return invokeObjcSelf!(NSString, "dataRefType");
	}

	NSString referenceFile ()
	{
		return invokeObjcSelf!(NSString, "referenceFile");
	}

	NSURL referenceURL ()
	{
		return invokeObjcSelf!(NSURL, "referenceURL");
	}

	NSData referenceData ()
	{
		return invokeObjcSelf!(NSData, "referenceData");
	}

	NSString name ()
	{
		return invokeObjcSelf!(NSString, "name");
	}

	NSString MIMEType ()
	{
		return invokeObjcSelf!(NSString, "MIMEType");
	}

	void setDataRef (Handle dataRef)
	{
		return invokeObjcSelf!(void, "setDataRef:", Handle)(dataRef);
	}

	void setDataRefType (NSString type)
	{
		return invokeObjcSelf!(void, "setDataRefType:", NSString)(type);
	}
	
	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}
	
	QTDataReference initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(QTDataReference, "initWithCoder:", NSCoder)(aDecoder);
	}
}

